@@ -19,11 +19,21 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 #include "../headers.h"
+
 #ifdef __unix__
 #include <GL/glx.h>
 #include <SDL/SDL_syswm.h>
 #endif
 
+#ifndef __unix__
+extern char *strtok_r(char *s1, const char *s2, char **lasts);
+#include <stdint.h>
+#define drand48 rand
+#endif
+
+int acc = 0;
+int maxfps = 60;
+
 Graphics::Graphics()
 {
 	fontIndex = 0;
@@ -58,8 +68,8 @@ Graphics::Graphics()
 	screenMode[3].w = 1280;
 	screenMode[3].h = 1024;
 	
-	screenMode[4].w = 1900;
-	screenMode[4].h = 1200;
+	screenMode[4].w = 1680;
+	screenMode[4].h = 1050;
 	
 	animTimer = 0;
 	
@@ -133,6 +143,24 @@ void Graphics::setMode(int mode)
 		glLoadIdentity();
 		
 		gluPerspective(45.0f, (GLfloat) screen->w / (GLfloat) screen->h, 0.1f, 9000.0f);
+		
+		// AA using the accumulation buffer
+
+		static GLfloat dx[100] = {-0.5, 0.5, -0.5, 0.5, 0, -0.5, 0.5, 0, 0, 0}; 
+		static GLfloat dy[100] = {-0.5, 0.5, 0.5, -0.5, 0, 0, 0, -0.5, 0.5, 0};
+
+		if(!dx[9]) {
+			for(int i = 9; i < 100; i++) {
+				dx[i] = drand48() - 1.0;
+				dy[i] = drand48() - 1.0;
+			}
+		}
+
+		GLfloat m[16];
+		glGetFloatv(GL_PROJECTION_MATRIX, m);
+		m[8] = dx[acc % 100]/(GLfloat)screen->w;
+		m[9] = dy[acc % 100]/(GLfloat)screen->h;
+		glLoadMatrixf(m);
 	
 		glMatrixMode(GL_MODELVIEW);
 		glLoadIdentity();
@@ -247,9 +275,12 @@ void Graphics::setResolution(int i)
 	debug(("Graphics::setResolution() - Done\n"));
 }
 
+static int swapframes = 0;
+
 void Graphics::resetFPSCount()
 {
 	frames = 0;
+	swapframes = 0;
 	time = 0.0;
 	fps = "N/A";
 }
@@ -267,12 +298,13 @@ void Graphics::drawFPSCount(int y)
 			
 	if (time <= 0)
 	{
-		fps.setText("%d fps", frames);
+		fps.setText("%d * %3.1f fps", swapframes, (float)frames / swapframes);
 		time = 100;
 		frames = 0;
+		swapframes = 0;
 	}
 	
-	drawString(screen->w - 5, y, TXT_RIGHT, GLColor::white, true, "%s", fps.getText());
+	drawString(screen->w - 11, y, TXT_RIGHT, GLColor::white, true, "%s", fps.getText());
 }
 
 void Graphics::showLoadingProgress(const char *status)
@@ -333,8 +365,30 @@ void Graphics::clearScreen()
 
 void Graphics::updateScreen()
 {
-	SDL_GL_SwapBuffers();
-	SDL_Delay(1);
+	static const int maxacc = 256;
+	bool flush = false;
+	static uint32_t lastflush = 0;
+	uint32_t now = SDL_GetTicks();
+
+	acc++;
+
+	// Decide to flush accumulation buffer if another frame would make us miss the next vblank.
+	if(now - lastflush >= 1000 / maxfps || acc >= maxacc)
+		flush = true;
+
+	// Accumulate this frame unless it's the first frame and we're flushing.
+	if(!flush || acc > 1)
+		glAccum(acc > 1 ? GL_ACCUM : GL_LOAD, 1.0f / maxacc);
+
+	if(flush) {
+		if(acc > 1)
+			glAccum(GL_RETURN, (float)maxacc / (float)acc);
+		acc = 0;
+		lastflush = now;
+		swapframes++;
+		SDL_GL_SwapBuffers();
+	}
+	//SDL_Delay(1);
 
 	if ((engine->keyState[SDLK_F10]) || ((engine->keyState[SDLK_RETURN]) && (engine->keyState[SDLK_LALT])))
 	{
@@ -587,7 +641,7 @@ Texture *Graphics::createTexture(SDL_Surface *surface)
 	glBindTexture(GL_TEXTURE_2D, glTexture);
 	
 	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tmp->pixels);
-	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, tmp->w, tmp->h, GL_RGBA, GL_UNSIGNED_BYTE, tmp->pixels);
 	
@@ -611,7 +665,7 @@ Texture *Graphics::createTextureFromTGA(SDL_Surface *surface)
 	
 	glGenTextures(1, &glTexture);
 	glBindTexture(GL_TEXTURE_2D, glTexture);
-	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	
 	Texture *texture = new Texture();
