@@ -432,6 +432,7 @@ void doCheatOptions()
 
 void doAdditionalOptions()
 {
+	extern int maxfps;
 	uiManager->onlyShowGroup("additional");
 	
 	Grid *grid = uiManager->getGrid("AdditionalGrid");
@@ -442,6 +443,7 @@ void doAdditionalOptions()
 	ComboBox *shadows = (ComboBox*)uiManager->getWidget("Shadows", "additional");
 	ComboBox *decals = (ComboBox*)uiManager->getWidget("Decals", "additional");
 	CheckBox *swap = (CheckBox*)uiManager->getWidget("SwapAxis", "additional");
+	Slider *fpslimit = (Slider*)uiManager->getWidget("FPSlimit", "additional");
 	
 	Button *done = (Button*)uiManager->getWidget("Done", "additional");
 	
@@ -472,6 +474,8 @@ void doAdditionalOptions()
 	
 	swap->checked = engine->swapSecondStickAxis;
 	
+	fpslimit->value = maxfps;
+
 	engine->resetTimeDifference();
 	
 	while (!finished)
@@ -508,6 +512,7 @@ void doAdditionalOptions()
 	game->decalPolicy = decals->selectedItem;
 	game->shadowPolicy = shadows->selectedItem;
 	engine->swapSecondStickAxis = swap->checked;
+	maxfps = fpslimit->value;
 	
 	blood->clear();
 	decals->clear();
