@@ -60,6 +60,13 @@ void removeSpecialWeapons()
 			game->weapon[i] = NULL;
 		}
 	}
+	
+	// June 2009 fix for carrying items between levels
+	for (int i = 0 ; i < 5 ; i++)
+	{
+		game->item[i] = NULL;
+		game->itemId[i] = 0;
+	}
 }
 
 WeaponDef *getRandomPlayerAmmoType()
